# Generated by roxygen2: do not edit by hand

export(cleanSeqs)
export(findNovelAlleles)
export(findUnmutatedCalls)
export(generateEvidence)
export(genotypeFasta)
export(getMutCount)
export(getMutatedPositions)
export(getPopularMutationCount)
export(inferGenotype)
export(inferGenotypeBayesian)
export(insertPolymorphisms)
export(plotGenotype)
export(plotNovel)
export(readIgFasta)
export(reassignAlleles)
export(selectNovel)
export(sortAlleles)
export(subsampleDb)
export(updateAlleleNames)
export(writeFasta)
import(ggplot2)
importFrom(alakazam,DNA_COLORS)
importFrom(alakazam,checkColumns)
importFrom(alakazam,getAllele)
importFrom(alakazam,getFamily)
importFrom(alakazam,getGene)
importFrom(alakazam,translateDNA)
importFrom(doParallel,registerDoParallel)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,as_data_frame)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,combine)
importFrom(dplyr,data_frame)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,do)
importFrom(dplyr,filter)
importFrom(dplyr,glimpse)
importFrom(dplyr,group_by)
importFrom(dplyr,group_indices)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(foreach,registerDoSEQ)
importFrom(graphics,plot)
importFrom(gridExtra,arrangeGrob)
importFrom(gtools,ddirichlet)
importFrom(iterators,icount)
importFrom(lazyeval,interp)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(stats,confint)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,ecdf)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringi,"stri_sub<-")
importFrom(stringi,stri_detect_fixed)
importFrom(stringi,stri_length)
importFrom(stringi,stri_replace_all_regex)
importFrom(stringi,stri_sub)
importFrom(stringi,stri_trans_toupper)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(tidyr,unnest)
importFrom(utils,citation)
